@@ -1,6 +1,6 @@
 /*
  * Copyright 2014-2019, CNRS
- * Copyright 2018-2019, INRIA
+ * Copyright 2018-2020, INRIA
  */
 
 #ifndef __eigenpy_details_hpp__
@@ -13,97 +13,25 @@
 #include <iostream>
 
 #include "eigenpy/eigenpy.hpp"
+#include "eigenpy/numpy-type.hpp"
 #include "eigenpy/registration.hpp"
 #include "eigenpy/map.hpp"
-
-namespace boost { namespace python { namespace detail {
-
-  template<class MatType>
-  struct referent_size<Eigen::MatrixBase<MatType>&>
-  {
-      BOOST_STATIC_CONSTANT(
-          std::size_t, value = sizeof(MatType));
-  };
-
-}}}
-
-namespace boost { namespace python { namespace converter {
-
-template<class MatType>
-struct implicit<Eigen::MatrixBase<MatType>,MatType>
-{
-  typedef Eigen::MatrixBase<MatType> Source;
-  typedef MatType Target;
-  
-  static void* convertible(PyObject* obj)
-  {
-    // Find a converter which can produce a Source instance from
-    // obj. The user has told us that Source can be converted to
-    // Target, and instantiating construct() below, ensures that
-    // at compile-time.
-    return implicit_rvalue_convertible_from_python(obj, registered<Source>::converters)
-    ? obj : 0;
-  }
-  
-  static void construct(PyObject* obj, rvalue_from_python_stage1_data* data)
-  {
-    void* storage = ((rvalue_from_python_storage<Target>*)data)->storage.bytes;
-    
-    arg_from_python<Source> get_source(obj);
-    bool convertible = get_source.convertible();
-    BOOST_VERIFY(convertible);
-    
-    new (storage) Target(get_source().derived());
-    
-    // record successful construction
-    data->convertible = storage;
-  }
-};
-
-template<class MatType>
-struct implicit<MatType,Eigen::MatrixBase<MatType> >
-{
-  typedef MatType Source;
-  typedef Eigen::MatrixBase<MatType> Target;
-  
-  static void* convertible(PyObject* obj)
-  {
-    // Find a converter which can produce a Source instance from
-    // obj. The user has told us that Source can be converted to
-    // Target, and instantiating construct() below, ensures that
-    // at compile-time.
-    return implicit_rvalue_convertible_from_python(obj, registered<Source>::converters)
-    ? obj : 0;
-  }
-  
-  static void construct(PyObject* obj, rvalue_from_python_stage1_data* data)
-  {
-    void* storage = reinterpret_cast<rvalue_from_python_storage<Target>*>
-                   (reinterpret_cast<void*>(data))->storage.bytes;
-    
-    arg_from_python<Source> get_source(obj);
-    bool convertible = get_source.convertible();
-    BOOST_VERIFY(convertible);
-    
-    new (storage) Source(get_source());
-    
-    // record successful construction
-    data->convertible = storage;
-  }
-};
-
-}}} // namespace boost::python::converter
+#include "eigenpy/exception.hpp"
 
 #define GET_PY_ARRAY_TYPE(array) PyArray_ObjectType(reinterpret_cast<PyObject *>(array), 0)
 
 namespace eigenpy
 {
   template <typename SCALAR>  struct NumpyEquivalentType {};
+  template <> struct NumpyEquivalentType<float>   { enum { type_code = NPY_FLOAT  };};
+  template <> struct NumpyEquivalentType< std::complex<float> >   { enum { type_code = NPY_CFLOAT  };};
   template <> struct NumpyEquivalentType<double>  { enum { type_code = NPY_DOUBLE };};
+  template <> struct NumpyEquivalentType< std::complex<double> >  { enum { type_code = NPY_CDOUBLE };};
+  template <> struct NumpyEquivalentType<long double>  { enum { type_code = NPY_LONGDOUBLE };};
+  template <> struct NumpyEquivalentType< std::complex<long double> >  { enum { type_code = NPY_CLONGDOUBLE };};
   template <> struct NumpyEquivalentType<int>     { enum { type_code = NPY_INT    };};
   template <> struct NumpyEquivalentType<long>    { enum { type_code = NPY_LONG    };};
-  template <> struct NumpyEquivalentType<float>   { enum { type_code = NPY_FLOAT  };};
-  
+
   template <typename SCALAR1, typename SCALAR2>
   struct FromTypeToType : public boost::false_type {};
   
@@ -112,154 +40,50 @@ namespace eigenpy
   
   template <> struct FromTypeToType<int,long> : public boost::true_type {};
   template <> struct FromTypeToType<int,float> : public boost::true_type {};
+  template <> struct FromTypeToType<int,std::complex<float> > : public boost::true_type {};
   template <> struct FromTypeToType<int,double> : public boost::true_type {};
+  template <> struct FromTypeToType<int,std::complex<double> > : public boost::true_type {};
+  template <> struct FromTypeToType<int,long double> : public boost::true_type {};
+  template <> struct FromTypeToType<int,std::complex<long double> > : public boost::true_type {};
   
   template <> struct FromTypeToType<long,float> : public boost::true_type {};
+  template <> struct FromTypeToType<long,std::complex<float> > : public boost::true_type {};
   template <> struct FromTypeToType<long,double> : public boost::true_type {};
+  template <> struct FromTypeToType<long,std::complex<double> > : public boost::true_type {};
+  template <> struct FromTypeToType<long,long double> : public boost::true_type {};
+  template <> struct FromTypeToType<long,std::complex<long double> > : public boost::true_type {};
   
+  template <> struct FromTypeToType<float,std::complex<float> > : public boost::true_type {};
   template <> struct FromTypeToType<float,double> : public boost::true_type {};
+  template <> struct FromTypeToType<float,std::complex<double> > : public boost::true_type {};
+  template <> struct FromTypeToType<float,long double> : public boost::true_type {};
+  template <> struct FromTypeToType<float,std::complex<long double> > : public boost::true_type {};
+
+  template <> struct FromTypeToType<double,std::complex<double> > : public boost::true_type {};
+  template <> struct FromTypeToType<double,long double> : public boost::true_type {};
+  template <> struct FromTypeToType<double,std::complex<long double> > : public boost::true_type {};
 
   namespace bp = boost::python;
 
-  enum NP_TYPE
-  {
-    DEFAULT_TYPE,
-    MATRIX_TYPE,
-    ARRAY_TYPE
-  };
-  
-  struct NumpyType
-  {
-    
-    static NumpyType & getInstance()
-    {
-      static NumpyType instance;
-      return instance;
-    }
-
-    operator bp::object () { return CurrentNumpyType; }
-
-    bp::object make(PyArrayObject* pyArray, bool copy = false)
-    { return make((PyObject*)pyArray,copy); }
-    
-    bp::object make(PyObject* pyObj, bool copy = false)
-    {
-      if (getType() == DEFAULT_TYPE) {
-        std::cerr <<
-          "eigenpy warning: you use the deprecated class numpy.matrix without explicily asking for it. "
-          "The default behaviour will change to numpy.array at next major release.\n"
-          "- Either call eigenpy.switchToNumpyMatrix() before using eigenpy to suppress this warning\n"
-          "- or call eigenpy.switchToNumpyArray() and adapt your code accordingly.\n"
-          "See https://github.com/stack-of-tasks/eigenpy/issues/87 for further details."
-          << std::endl;
-        switchToNumpyMatrix();
-      }
-      bp::object m;
-      if(PyType_IsSubtype(reinterpret_cast<PyTypeObject*>(CurrentNumpyType.ptr()),NumpyMatrixType))
-        m = NumpyMatrixObject(bp::object(bp::handle<>(pyObj)), bp::object(), copy);
-//        m = NumpyAsMatrixObject(bp::object(bp::handle<>(pyObj)));
-      else if(PyType_IsSubtype(reinterpret_cast<PyTypeObject*>(CurrentNumpyType.ptr()),NumpyArrayType))
-        m = bp::object(bp::handle<>(pyObj)); // nothing to do here
-
-      Py_INCREF(m.ptr());
-      return m;
-    }
-    
-    static void setNumpyType(bp::object & obj)
-    {
-      PyTypeObject * obj_type = PyType_Check(obj.ptr()) ? reinterpret_cast<PyTypeObject*>(obj.ptr()) : obj.ptr()->ob_type;
-      if(PyType_IsSubtype(obj_type,getInstance().NumpyMatrixType))
-        switchToNumpyMatrix();
-      else if(PyType_IsSubtype(obj_type,getInstance().NumpyArrayType))
-        switchToNumpyArray();
-    }
-    
-    static void switchToNumpyArray()
-    {
-      getInstance().CurrentNumpyType = getInstance().NumpyArrayObject;
-      getType() = ARRAY_TYPE;
-    }
-    
-    static void switchToNumpyMatrix()
-    {
-      getInstance().CurrentNumpyType = getInstance().NumpyMatrixObject;
-      getType() = MATRIX_TYPE;
-    }
-    
-    static NP_TYPE & getType()
-    {
-      static NP_TYPE np_type;
-      return np_type;
-    }
-    
-    static bp::object getNumpyType()
-    {
-      return getInstance().CurrentNumpyType;
-    }
-    
-    static const PyTypeObject * getNumpyMatrixType()
-    {
-      return getInstance().NumpyMatrixType;
-    }
-    
-    static const PyTypeObject * getNumpyArrayType()
-    {
-      return getInstance().NumpyArrayType;
-    }
-    
-    static bool isMatrix()
-    {
-      return PyType_IsSubtype(reinterpret_cast<PyTypeObject*>(getInstance().CurrentNumpyType.ptr()),
-                              getInstance().NumpyMatrixType);
-    }
-    
-    static bool isArray()
-    {
-      return PyType_IsSubtype(reinterpret_cast<PyTypeObject*>(getInstance().CurrentNumpyType.ptr()),
-                              getInstance().NumpyArrayType);
-    }
-
-  protected:
-    NumpyType()
-    {
-      pyModule = bp::import("numpy");
-#if PY_MAJOR_VERSION >= 3
-      // TODO I don't know why this Py_INCREF is necessary.
-      // Without it, the destructor of NumpyType SEGV sometimes.
-      Py_INCREF(pyModule.ptr());
-#endif
-      
-      NumpyMatrixObject = pyModule.attr("matrix");
-      NumpyMatrixType = reinterpret_cast<PyTypeObject*>(NumpyMatrixObject.ptr());
-      NumpyArrayObject = pyModule.attr("ndarray");
-      NumpyArrayType = reinterpret_cast<PyTypeObject*>(NumpyArrayObject.ptr());
-      //NumpyAsMatrixObject = pyModule.attr("asmatrix");
-      //NumpyAsMatrixType = reinterpret_cast<PyTypeObject*>(NumpyAsMatrixObject.ptr());
-      
-      CurrentNumpyType = NumpyMatrixObject; // default conversion
-      getType() = DEFAULT_TYPE;
-    }
-
-    bp::object CurrentNumpyType;
-    bp::object pyModule;
-    
-    // Numpy types
-    bp::object NumpyMatrixObject; PyTypeObject * NumpyMatrixType;
-    //bp::object NumpyAsMatrixObject; PyTypeObject * NumpyAsMatrixType;
-    bp::object NumpyArrayObject; PyTypeObject * NumpyArrayType;
-    
-  };
-
   template<typename MatType, bool IsVectorAtCompileTime = MatType::IsVectorAtCompileTime>
   struct initEigenObject
   {
     static MatType * run(PyArrayObject * pyArray, void * storage)
     {
-      assert(PyArray_NDIM(pyArray) == 2);
+      assert(PyArray_NDIM(pyArray) == 1 || PyArray_NDIM(pyArray) == 2);
 
-      const int rows = (int)PyArray_DIMS(pyArray)[0];
-      const int cols = (int)PyArray_DIMS(pyArray)[1];
-      
+      int rows = -1, cols = -1;
+      if(PyArray_NDIM(pyArray) == 2)
+      {
+        rows = (int)PyArray_DIMS(pyArray)[0];
+        cols = (int)PyArray_DIMS(pyArray)[1];
+      }
+      else if(PyArray_NDIM(pyArray) == 1)
+      {
+        rows = (int)PyArray_DIMS(pyArray)[0];
+        cols = 1;
+      }
+              
       return new (storage) MatType(rows,cols);
     }
   };
@@ -282,6 +106,39 @@ namespace eigenpy
       }
     }
   };
+
+  template<typename Scalar, typename NewScalar, bool cast_is_valid = FromTypeToType<Scalar,NewScalar>::value >
+  struct CastMatToMat
+  {
+    template<typename MatrixIn, typename MatrixOut>
+    static void run(const Eigen::MatrixBase<MatrixIn> & input,
+                    const Eigen::MatrixBase<MatrixOut> & dest)
+    {
+      MatrixOut & dest_ = const_cast<MatrixOut &>(dest.derived());
+      if(dest.rows() == input.rows())
+        dest_ = input.template cast<NewScalar>();
+      else
+        dest_ = input.transpose().template cast<NewScalar>();
+    }
+  };
+
+  template<typename Scalar, typename NewScalar>
+  struct CastMatToMat<Scalar,NewScalar,false>
+  {
+    template<typename MatrixIn, typename MatrixOut>
+    static void run(const Eigen::MatrixBase<MatrixIn> & /*input*/,
+                    const Eigen::MatrixBase<MatrixOut> & /*dest*/)
+    {
+      // do nothing
+      assert("Must never happened");
+    }
+  };
+
+#define EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,Scalar,NewScalar,pyArray,mat) \
+  CastMatToMat<Scalar,NewScalar>::run(MapNumpy<MatType,Scalar>::map(pyArray),mat)
+
+#define EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,NewScalar,mat,pyArray) \
+  CastMatToMat<Scalar,NewScalar>::run(mat,MapNumpy<MatType,NewScalar>::map(pyArray))
   
   template<typename MatType>
   struct EigenObjectAllocator
@@ -292,35 +149,43 @@ namespace eigenpy
     static void allocate(PyArrayObject * pyArray, void * storage)
     {
       Type * mat_ptr = initEigenObject<Type>::run(pyArray,storage);
+      Type & mat = *mat_ptr;
       
-      if(NumpyEquivalentType<Scalar>::type_code == GET_PY_ARRAY_TYPE(pyArray))
+      const int pyArray_Type = GET_PY_ARRAY_TYPE(pyArray);
+      if(pyArray_Type == NumpyEquivalentType<Scalar>::type_code)
       {
-        *mat_ptr = MapNumpy<MatType,Scalar>::map(pyArray); // avoid useless cast
+        mat = MapNumpy<MatType,Scalar>::map(pyArray); // avoid useless cast
         return;
       }
       
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_INT)
+      switch(pyArray_Type)
       {
-        *mat_ptr = MapNumpy<MatType,int>::map(pyArray).template cast<Scalar>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_LONG)
-      {
-        *mat_ptr = MapNumpy<MatType,long>::map(pyArray).template cast<Scalar>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_FLOAT)
-      {
-        *mat_ptr = MapNumpy<MatType,float>::map(pyArray).template cast<Scalar>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_DOUBLE)
-      {
-        *mat_ptr = MapNumpy<MatType,double>::map(pyArray).template cast<Scalar>();
-        return;
+        case NPY_INT:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,int,Scalar,pyArray,mat);
+          break;
+        case NPY_LONG:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,long,Scalar,pyArray,mat);
+          break;
+        case NPY_FLOAT:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,float,Scalar,pyArray,mat);
+          break;
+        case NPY_CFLOAT:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,std::complex<float>,Scalar,pyArray,mat);
+          break;
+        case NPY_DOUBLE:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,double,Scalar,pyArray,mat);
+          break;
+        case NPY_CDOUBLE:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,std::complex<double>,Scalar,pyArray,mat);
+          break;
+        case NPY_LONGDOUBLE:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,long double,Scalar,pyArray,mat);
+          break;
+        case NPY_CLONGDOUBLE:
+          EIGENPY_CAST_FROM_PYARRAY_TO_EIGEN_MATRIX(MatType,std::complex<long double>,Scalar,pyArray,mat);
+          break;
+        default:
+          throw Exception("You asked for a conversion which is not implemented.");
       }
     }
     
@@ -330,35 +195,48 @@ namespace eigenpy
                      PyArrayObject * pyArray)
     {
       const MatrixDerived & mat = const_cast<const MatrixDerived &>(mat_.derived());
+      const int pyArray_Type = GET_PY_ARRAY_TYPE(pyArray);
       
-      if(NumpyEquivalentType<Scalar>::type_code == GET_PY_ARRAY_TYPE(pyArray))
+      typedef typename MapNumpy<MatType,Scalar>::EigenMap MapType;
+      
+      if(pyArray_Type == NumpyEquivalentType<Scalar>::type_code) // no cast needed
       {
-        MapNumpy<MatType,Scalar>::map(pyArray) = mat; // no cast needed
+        MapType map_pyArray = MapNumpy<MatType,Scalar>::map(pyArray);
+        if(mat.rows() == map_pyArray.rows())
+          map_pyArray = mat;
+        else
+          map_pyArray = mat.transpose();
         return;
       }
       
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_INT)
+      switch(pyArray_Type)
       {
-        MapNumpy<MatType,int>::map(pyArray) = mat.template cast<int>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_LONG)
-      {
-        MapNumpy<MatType,long>::map(pyArray) = mat.template cast<long>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_FLOAT)
-      {
-        MapNumpy<MatType,float>::map(pyArray) = mat.template cast<float>();
-        return;
-      }
-      
-      if(GET_PY_ARRAY_TYPE(pyArray) == NPY_DOUBLE)
-      {
-        MapNumpy<MatType,double>::map(pyArray) = mat.template cast<double>();
-        return;
+        case NPY_INT:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,int,mat,pyArray);
+          break;
+        case NPY_LONG:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,long,mat,pyArray);
+          break;
+        case NPY_FLOAT:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,float,mat,pyArray);
+          break;
+        case NPY_CFLOAT:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,std::complex<float>,mat,pyArray);
+          break;
+        case NPY_DOUBLE:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,double,mat,pyArray);
+          break;
+        case NPY_CDOUBLE:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,std::complex<double>,mat,pyArray);
+          break;
+        case NPY_LONGDOUBLE:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,long double,mat,pyArray);
+          break;
+        case NPY_CLONGDOUBLE:
+          EIGENPY_CAST_FROM_EIGEN_MATRIX_TO_PYARRAY(MatType,Scalar,std::complex<long double>,mat,pyArray);
+          break;
+        default:
+          throw Exception("You asked for a conversion which is not implemented.");
       }
     }
   };
@@ -393,13 +271,14 @@ namespace eigenpy
       typedef typename MatType::Scalar Scalar;
       assert( (mat.rows()<INT_MAX) && (mat.cols()<INT_MAX) 
 	      && "Matrix range larger than int ... should never happen." );
-      const int R  = (int)mat.rows(), C = (int)mat.cols();
+      const npy_intp R = (npy_intp)mat.rows(), C = (npy_intp)mat.cols();
 
       PyArrayObject* pyArray;
       // Allocate Python memory
-      if(C == 1 && NumpyType::getType() == ARRAY_TYPE) // Handle array with a single dimension
+      if( ( ((!(C == 1) != !(R == 1)) && !MatType::IsVectorAtCompileTime) || MatType::IsVectorAtCompileTime)
+         && NumpyType::getType() == ARRAY_TYPE) // Handle array with a single dimension
       {
-        npy_intp shape[1] = { R };
+        npy_intp shape[1] = { C == 1 ? R : C };
         pyArray = (PyArrayObject*) PyArray_SimpleNew(1, shape,
                                                      NumpyEquivalentType<Scalar>::type_code);
       }
@@ -437,8 +316,16 @@ namespace eigenpy
           return FromTypeToType<long,typename MatType::Scalar>::value;
         case NPY_FLOAT:
           return FromTypeToType<float,typename MatType::Scalar>::value;
+        case NPY_CFLOAT:
+          return FromTypeToType<std::complex<float>,typename MatType::Scalar>::value;
         case NPY_DOUBLE:
           return FromTypeToType<double,typename MatType::Scalar>::value;
+        case NPY_CDOUBLE:
+          return FromTypeToType<std::complex<double>,typename MatType::Scalar>::value;
+        case NPY_LONGDOUBLE:
+          return FromTypeToType<long double,typename MatType::Scalar>::value;
+        case NPY_CLONGDOUBLE:
+          return FromTypeToType<std::complex<long double>,typename MatType::Scalar>::value;
         default:
           return false;
       }
@@ -530,17 +417,19 @@ namespace eigenpy
       }
       else // this is a matrix
       {
+        if(PyArray_NDIM(pyArray) == 1) // We can always convert a vector into a matrix
+        {
+          return pyArray;
+        }
+        
         if(PyArray_NDIM(pyArray) != 2)
         {
-          if ( (PyArray_NDIM(pyArray) !=1) || (! MatType::IsVectorAtCompileTime) )
-          {
 #ifndef NDEBUG
             std::cerr << "The number of dimension of the object is not correct." << std::endl;
 #endif
-            return 0;
-          }
+          return 0;
         }
-        
+       
         if(PyArray_NDIM(pyArray) == 2)
         {
           const int R = (int)PyArray_DIMS(pyArray)[0];
@@ -608,9 +497,40 @@ namespace eigenpy
       EigenFromPy<MatType>::registration();
 
       // Add also conversion to Eigen::MatrixBase<MatType>
-      typedef Eigen::MatrixBase<MatType> MatTypeBase;
-//      bp::implicitly_convertible<MatTypeBase,MatType>();
-      bp::implicitly_convertible<MatType,MatTypeBase>();
+      typedef Eigen::MatrixBase<MatType> MatrixBase;
+      EigenFromPy<MatrixBase>::registration();
+
+      // Add also conversion to Eigen::EigenBase<MatType>
+      typedef Eigen::EigenBase<MatType> EigenBase;
+      EigenFromPy<EigenBase>::registration();
+    }
+  };
+
+  template<typename MatType>
+  struct EigenFromPy< Eigen::MatrixBase<MatType> > : EigenFromPy<MatType>
+  {
+    typedef EigenFromPy<MatType> EigenFromPyDerived;
+    typedef Eigen::MatrixBase<MatType> Base;
+
+    static void registration()
+    {
+      bp::converter::registry::push_back
+      (reinterpret_cast<void *(*)(_object *)>(&EigenFromPy::convertible),
+       &EigenFromPy::construct,bp::type_id<Base>());
+    }
+  };
+    
+  template<typename MatType>
+  struct EigenFromPy< Eigen::EigenBase<MatType> > : EigenFromPy<MatType>
+  {
+    typedef EigenFromPy<MatType> EigenFromPyDerived;
+    typedef Eigen::EigenBase<MatType> Base;
+
+    static void registration()
+    {
+      bp::converter::registry::push_back
+      (reinterpret_cast<void *(*)(_object *)>(&EigenFromPy::convertible),
+       &EigenFromPy::construct,bp::type_id<Base>());
     }
   };
 
